#include <Image.CompoundCoordinatesConverter.h>
#include <Image.TransformationFunctions.h>
#include <Image.GeometricalDistortionsRemovalParams.h>
#include <Image.PerspectiveRemovalParams.h>
#include <Image.MotionBlurRemovalParams.h>
#include <Image.ISONoiseFilteringParams.h>
#include <Image.ReceiptDewarpingParams.h>
#include <Image.PrepareReceiptParams.h>
#include <Image.Region.h>
#include <Image.ReceiptFunctions.h>

namespace Image {

inline CPtr<const IImage> RemoveGeometricalDistortions( const IImage* sourceImage,
	const IGeometricalDistortionsRemovalParams* params,
	ICompoundCoordinatesConverter* converter )
{
	return ApplyAlgorithm( sourceImage, GeometricalDistortionsRemoverClass,
		params, converter );
}

inline CPtr<const IImage> RemovePerspectiveDistortions( const IImage* sourceImage,
	const IPerspectiveRemovalParams* params, ICompoundCoordinatesConverter* converter )
{
	return ApplyAlgorithm( sourceImage, PerspectiveRemovalTransformerClass,
		params, converter );
}

inline CPtr<const IImage> RemovePerspectiveDistortions( const IImage* sourceImage,
	const IPerspectiveRemovalParams* params, CSize resultImageSize, ICompoundCoordinatesConverter* converter )
{
	const_cast<IPerspectiveRemovalParams*>( params )->SetResultImageSize( resultImageSize );
	return ApplyAlgorithm( sourceImage, PerspectiveRemovalTransformerClass,
		params, converter );
}

inline CPtr<const IImage> RemoveMotionBlur( const IImage* sourceImage,
	const IRegion* noiseDetectionRegion, ICompoundCoordinatesConverter* converter )
{
	CPtr<IMotionBlurRemovalParams> params = FObj::CreateObject<IMotionBlurRemovalParams>(
		MotionBlurRemovalParamsClass );
	params->SetNoiseDetectionRegion( noiseDetectionRegion );

	return ApplyAlgorithm( sourceImage, MotionBlurRemoverClass, params, converter );
}

inline CPtr<const IImage> FilterISONoise( const IImage* sourceImage,
	const IRegion* noiseDetectionRegion, ICompoundCoordinatesConverter* converter )
{
	CPtr<IISONoiseFilteringParams> params = FObj::CreateObject<IISONoiseFilteringParams>(
		ISONoiseFilteringParamsClass );
	params->SetNoiseDetectionRegion( noiseDetectionRegion );

	return ApplyAlgorithm( sourceImage, ISONoiseFilterClass, params, converter );
}

inline CPtr<const IImage> EqualizeBrightnessOnImage( const IImage* sourceImage,
	const DocumentProcessing::ILayout* layout, TBrightnessEqualizationType type,
	ICompoundCoordinatesConverter* converter )
{
	CPtr<IBrightnessEqualizationWithTextInfoParams> params = FObj::CreateObject<IBrightnessEqualizationWithTextInfoParams>(
		BrightnessEqualizationWithTextInfoParamsClass );
	params->SetLayout( layout );
	params->SetType( type );

	return ApplyAlgorithm( sourceImage, BrightnessEqualizationWithTextInfoTransformerClass,
		params, converter );
}

inline CPtr<const IImage> RemoveScannerShadowFromImage( const IImage* sourceImage,
	ICompoundCoordinatesConverter* converter )
{
	return ApplyAlgorithm( sourceImage, ScannerShadowRemoverClass, 0, converter );
}

inline CPtr<const IImage> DewarpReceipt( const IImage* sourceImage, 
	const IReceiptDewarpingParams* params, ICompoundCoordinatesConverter* converter )
{
	return ApplyAlgorithm( sourceImage, ReceiptDewarperClass, params, converter );
}

inline CPtr<const IImage> PrepareReceipt( const IImage* sourceImage,
	const IPrepareReceiptParams* params, IRegion* cropRegion, ICompoundCoordinatesConverter* converter )
{
	return FObj::CreateObject<IImageReceiptFunctions>(
		ReceiptFunctionsClass )->PrepareReceipt( sourceImage, params, cropRegion, converter );
}

} // namespace Image