
//	Copyright (c) ABBYY (BIT Software).
//	:		
//	:	FineReader
//	:	Image
//	:	 inline-,   <Image.ColorManagement.h>.

#pragma once

namespace Image {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

static const int CoeffNormalizationBitsCount = 15;
static const int CoeffNormalization = 1 << CoeffNormalizationBitsCount;

static const int LumaRed = 9798;	//	0.299
static const int LumaGreen = 19235;	//	0.587
static const int LumaBlue = 3735;	//	0.114
compileTimeAssert( LumaRed + LumaGreen + LumaBlue == CoeffNormalization );	//	LumaRed + LumaGreen + LumaBlue = 1.0

//	  RGB->YCC
//	Y = LumaRed * R + LumaGreen * G + LumaBlue * B
//	Cb = 128 + (B - Y) / (2 - 2 * LumaBlue) = 128 + 0.5643 (B - Y) = 
//	   = 128 - LumaRed / (2 - 2 LumaBlue) R - LumaGreen / (2 - 2 LumaBlue) G + B / 2 = 128 - 0.1687 R - 0.3313 G + 0.5 B
//	Cr = 128 + (R - Y) / (2 - 2 * LumaRed) = 128 + 0.7133 (R - Y) = 
//	   = 128 + R / 2  - LumaGreen / (2 - 2 LumaRed) G - LumaBlue / (2 - 2 LumaRed) B = 128 + 0.5 R - 0.4187 G - 0.0813 B
static const int CoeffRgb2YccCbRed = -5529;	//	-0.1687
static const int CoeffRgb2YccCbGreen = -10855;	//	-0.3313
static const int CoeffRgb2YccCbBlue = 16384;	//	0.5
compileTimeAssert( CoeffRgb2YccCbRed + CoeffRgb2YccCbGreen + CoeffRgb2YccCbBlue == 0 );
static const int CoeffRgb2YccCrRed = 16384;	//	0.5
static const int CoeffRgb2YccCrGreen = -13720;	//	-0.4187
static const int CoeffRgb2YccCrBlue = -2664;	//	-0.0813
compileTimeAssert( CoeffRgb2YccCrRed + CoeffRgb2YccCrGreen + CoeffRgb2YccCrBlue == 0 );

//	  YCC->RGB
//	R = Y + (2 - 2 * LumaRed) * (Cr - 128) = Y + 1.402 (Cr - 128)
//	G = Y - ( (2 - 2 * LumaBlue) * LumaBlue / LumaGreen ) * (Cb - 128) - ( (2 - 2 * LumaRed) * LumaRed / LumaGreen ) * (Cr - 128) =
//	  = Y - 0.34414 (Cb - 128) - 0.71414 (Cr - 128)
//	B = Y + (2 - 2 * LumaBlue) * (Cb - 128) = Y + 1.772 (Cb - 128)
static const int CoeffYcc2RgbRedCr = 45940; // 2 * (CoeffNormalization - LumaRed);
static const int CoeffYcc2RgbGreenCb = -11275; // -Round( 2 * (CoeffNormalization - LumaBlue) * rational(LumaBlue, LumaGreen) );
static const int CoeffYcc2RgbGreenCr = -23401;	// -Round( 2 * (CoeffNormalization - LumaRed) * rational(LumaRed, LumaGreen) );
static const int CoeffYcc2RgbBlueCb = 58066; // 2 * (CoeffNormalization - LumaBlue);

static const int RgbToHslFactor = 1398101;	// Round( 256 * CoeffNormalization, 6 );

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

template<int COMPONENTS_COUNT>
CArchive& operator << (CArchive& arc, const CColor<COMPONENTS_COUNT>& color)
{
	for( int i = 0; i < COMPONENTS_COUNT; i++ ) {
		arc << color.Components[i];
	}
	return arc;
}

template<int COMPONENTS_COUNT>
CArchive& operator >> (CArchive& arc, CColor<COMPONENTS_COUNT>& color)
{
	for( int i = 0; i < COMPONENTS_COUNT; i++ ) {
		arc >> color.Components[i];
	}
	return arc;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

inline TColorComponent CGrayColor::Value() const 
{ 	
	return Components[0]; 
}

inline TColorComponent& CGrayColor::Value() 
{ 
	return Components[0]; 
}

inline const TColorComponent* CGrayColor::GetRawPtr() const 
{ 
	return Components.GetPtr(); 
}

inline TColorComponent* CGrayColor::GetRawPtr()
{ 
	return Components.GetPtr(); 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

inline TColorComponent	CRGBColor::Blue() const 
{ 
	return Components[RGBCCI_Blue]; 
}

inline TColorComponent& CRGBColor::Blue() 
{ 
	return Components[RGBCCI_Blue]; 
}

inline TColorComponent CRGBColor::Green() const 
{ 
	return Components[RGBCCI_Green];
}

inline TColorComponent& CRGBColor::Green() 
{ 
	return Components[RGBCCI_Green]; 
}

inline TColorComponent CRGBColor::Red() const 
{ 
	return Components[RGBCCI_Red]; 
}

inline TColorComponent& CRGBColor::Red() 
{ 
	return Components[RGBCCI_Red]; 
}

inline const TColorComponent* CRGBColor::GetRawPtr() const 
{
	compileTimeAssert( RGBCCI_EnumSize == 3 );
	compileTimeAssert( RGBCCI_Blue == 0 );
	compileTimeAssert( RGBCCI_Green == 1 );
	compileTimeAssert( RGBCCI_Red == 2 );

	return Components.GetPtr(); 
}

inline TColorComponent* CRGBColor::GetRawPtr()
{ 
	compileTimeAssert( RGBCCI_EnumSize == 3 );
	compileTimeAssert( RGBCCI_Blue == 0 );
	compileTimeAssert( RGBCCI_Green == 1 );
	compileTimeAssert( RGBCCI_Red == 2 );

	return Components.GetPtr(); 
}

/////////////////////////////////////////////////////////////////////////////////////////
//  inline-  CRgbaColor.

inline TColorComponent CRgbaColor::Blue() const
{
	return Components[RGBACCI_Blue]; 
}

inline TColorComponent& CRgbaColor::Blue()
{
	return Components[RGBACCI_Blue]; 
}

inline TColorComponent CRgbaColor::Green() const
{
	return Components[RGBACCI_Green]; 
}

inline TColorComponent& CRgbaColor::Green()
{
	return Components[RGBACCI_Green]; 
}

inline TColorComponent CRgbaColor::Red() const
{
	return Components[RGBACCI_Red]; 
}

inline TColorComponent& CRgbaColor::Red()
{
	return Components[RGBACCI_Red]; 
}

inline TColorComponent CRgbaColor::Alpha() const
{
	return Components[RGBACCI_Alpha]; 
}

inline TColorComponent& CRgbaColor::Alpha()
{
	return Components[RGBACCI_Alpha]; 
}

/////////////////////////////////////////////////////////////////////////////////////////

inline TColorComponent CYCbCrColor::Luminance() const 
{ 
	return Components[YCCCCI_Luminance]; 
}

inline TColorComponent& CYCbCrColor::Luminance() 
{ 
	return Components[YCCCCI_Luminance]; 
}

inline TColorComponent CYCbCrColor::ChrominanceBlue() const 
{ 
	return Components[YCCCCI_ChrominanceBlue]; 
}

inline TColorComponent& CYCbCrColor::ChrominanceBlue() 
{ 
	return Components[YCCCCI_ChrominanceBlue]; 
}

inline TColorComponent CYCbCrColor::ChrominanceRed() const 
{ 
	return Components[YCCCCI_ChrominanceRed]; 
}

inline TColorComponent& CYCbCrColor::ChrominanceRed() 
{
	return Components[YCCCCI_ChrominanceRed]; 
}

/////////////////////////////////////////////////////////////////////////////////////////

inline TColorComponent CHSLColorN::Hue() const 
{ 
	return Components[HSLCCI_Hue]; 
}

inline TColorComponent& CHSLColorN::Hue() 
{ 
	return Components[HSLCCI_Hue]; 
}

inline TColorComponent CHSLColorN::Saturation() const 
{ 
	return Components[HSLCCI_Saturation]; 
}

inline TColorComponent& CHSLColorN::Saturation() 
{ 
	return Components[HSLCCI_Saturation]; 
}

inline TColorComponent CHSLColorN::Lightness() const 
{ 
	return Components[HSLCCI_Lightness]; 
}

inline TColorComponent& CHSLColorN::Lightness() 
{
	return Components[HSLCCI_Lightness]; 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

inline TColorComponent CHSLColorNormalized::Hue() const 
{ 
	return Components[HSLCCI_Hue]; 
}

inline TColorComponent& CHSLColorNormalized::Hue() 
{ 
	return Components[HSLCCI_Hue]; 
}

inline TColorComponent CHSLColorNormalized::SaturationNormalized() const 
{ 
	return Components[HSLCCI_Saturation]; 
}

inline TColorComponent& CHSLColorNormalized::SaturationNormalized() 
{ 
	return Components[HSLCCI_Saturation]; 
}

inline TColorComponent CHSLColorNormalized::Lightness() const 
{ 
	return Components[HSLCCI_Lightness]; 
}

inline TColorComponent& CHSLColorNormalized::Lightness() 
{
	return Components[HSLCCI_Lightness]; 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

inline TColorComponent CCMYKColor::Cyan() const 
{ 
	return Components[CMYKCCI_Cyan];
}

inline TColorComponent& CCMYKColor::Cyan() 
{ 
	return Components[CMYKCCI_Cyan];
}

inline TColorComponent CCMYKColor::Magenta() const 
{ 
	return Components[CMYKCCI_Magenta]; 
}

inline TColorComponent& CCMYKColor::Magenta() 
{ 
	return Components[CMYKCCI_Magenta]; 
}

inline TColorComponent CCMYKColor::Yellow() const 
{ 
	return Components[CMYKCCI_Yellow]; 
}

inline TColorComponent& CCMYKColor::Yellow() 
{ 
	return Components[CMYKCCI_Yellow]; 
}

inline TColorComponent CCMYKColor::Key() const 
{ 
	return Components[CMYKCCI_Key]; 
}

inline TColorComponent& CCMYKColor::Key() 
{ 
	return Components[CMYKCCI_Key]; 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

inline TColorComponent CYCbCrKColor::Luminance() const 
{ 
	return Components[YCCKCCI_Luminance]; 
}

inline TColorComponent& CYCbCrKColor::Luminance() 
{ 
	return Components[YCCKCCI_Luminance]; 
}

inline TColorComponent CYCbCrKColor::ChrominanceBlue() const 
{ 
	return Components[YCCKCCI_ChrominanceBlue]; 
}

inline TColorComponent& CYCbCrKColor::ChrominanceBlue() 
{ 
	return Components[YCCKCCI_ChrominanceBlue]; 
}

inline TColorComponent CYCbCrKColor::ChrominanceRed() const 
{ 
	return Components[YCCKCCI_ChrominanceRed]; 
}

inline TColorComponent& CYCbCrKColor::ChrominanceRed() 
{ 
	return Components[YCCKCCI_ChrominanceRed]; 
}

inline TColorComponent CYCbCrKColor::Key() const 
{
	return Components[YCCKCCI_ChrominanceKey]; 
}

inline TColorComponent& CYCbCrKColor::Key() 
{ 
	return Components[YCCKCCI_ChrominanceKey]; 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
enum THSVColorIndices {
	HSVCI_HueIndex = 0,
	HSVCI_SaturationIndex = 1,
	HSVCI_ValueIndex = 2
};

inline TColorComponent CHSVColor::Hue() const
{
	return Components[HSVCI_HueIndex];
}

inline TColorComponent& CHSVColor::Hue()
{
	return Components[HSVCI_HueIndex];
}

inline TColorComponent CHSVColor::Saturation() const
{
	return Components[HSVCI_SaturationIndex];
}

inline TColorComponent& CHSVColor::Saturation()
{
	return Components[HSVCI_SaturationIndex];
}

inline TColorComponent CHSVColor::Value() const
{
	return Components[HSVCI_ValueIndex];
}

inline TColorComponent& CHSVColor::Value()
{
	return Components[HSVCI_ValueIndex];
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

#ifndef _FINAL

inline CUnicodeString GetColorString( COLORREF color )
{
	if( color == UndefinedColor ) {
		return L"Undefined";
	}
	if( color == TransparentColor ) {
		return L"Transparent";
	}
	return Format( L"(%3d,%3d,%3d)", 
		GetRValue( color ), GetGValue( color ), GetBValue( color ) );
}

#endif

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

inline void fromRGBToYCC( const CRGBColor& rgb, CYCbCrColor& ycc )
{
	//	-1 ,  255.5   255,  0.5   0
	static const int chrominanceAddition = 128 * CoeffNormalization + CoeffNormalization / 2 - 1;

	const int luminance = ( ( LumaRed * rgb.Red() + LumaGreen * rgb.Green() + LumaBlue * rgb.Blue() + CoeffNormalization / 2 ) >> 
		CoeffNormalizationBitsCount );
	const int chrominanceBlue = ( ( CoeffRgb2YccCbRed * rgb.Red() + CoeffRgb2YccCbGreen * rgb.Green() + 
		CoeffRgb2YccCbBlue * rgb.Blue() + chrominanceAddition ) >> CoeffNormalizationBitsCount );
	const int chrominanceRed = ( ( CoeffRgb2YccCrRed * rgb.Red() + CoeffRgb2YccCrGreen * rgb.Green() + 
		CoeffRgb2YccCrBlue * rgb.Blue() + chrominanceAddition ) >> CoeffNormalizationBitsCount );

	ycc.Luminance() = to<TColorComponent>( luminance );
	ycc.ChrominanceBlue() = to<TColorComponent>( chrominanceBlue );
	ycc.ChrominanceRed() = to<TColorComponent>( chrominanceRed );
}

//	,      (R, G  B)  , 
//	    scaleFactor. 
//	luminanceScaled  componentDeltaScaled   ,     component.
//	 true,    .
inline bool adjustScaleFactor( int component, int luminanceScaled, int componentDeltaScaled, double& scaleFactor )
{
	//	   -1  ColorComponentMax + 1  -  ,
	//	        0  ColorComponentMax,   .
	if( component < -1 ) {
		presume( componentDeltaScaled < 0 );
		const double factor = double( -luminanceScaled ) / componentDeltaScaled;
		presume( factor >= 0 );
		if( factor < scaleFactor ) {
			scaleFactor = factor;
			return true;
		}
	} else if( component > ColorComponentMax + 1 ) {
		presume( componentDeltaScaled > 0 );
		const double factor = double( ColorComponentMax * CoeffNormalization - luminanceScaled ) / componentDeltaScaled;
		presume( factor >= 0 );
		if( factor < scaleFactor ) {
			scaleFactor = factor;
			return true;
		}
	}
	return false;
}


//  YCC  RGB,    
inline void fromYCCToRGB( const CYCbCrColor& ycc, CRGBColor& rgb, double& scaleFactor )
{
	const int chrominanceBlueSigned = ycc.ChrominanceBlue() - 128;
	const int chrominanceRedSigned = ycc.ChrominanceRed() - 128;
	const int luminanceScaled = ycc.Luminance() * CoeffNormalization;

	const int redDeltaScaled = CoeffYcc2RgbRedCr * chrominanceRedSigned;
	const int greenDeltaScaled = CoeffYcc2RgbGreenCb * chrominanceBlueSigned + CoeffYcc2RgbGreenCr * chrominanceRedSigned;
	const int blueDeltaScaled = CoeffYcc2RgbBlueCb * chrominanceBlueSigned;

	int red = ( luminanceScaled + redDeltaScaled + CoeffNormalization / 2 ) >> CoeffNormalizationBitsCount;
	int green = ( luminanceScaled + greenDeltaScaled + CoeffNormalization / 2 ) >> CoeffNormalizationBitsCount;
	int blue = ( luminanceScaled + blueDeltaScaled + CoeffNormalization / 2 ) >> CoeffNormalizationBitsCount;

	bool scaleAdjusted = false;
	scaleFactor = 1.0;
	if( adjustScaleFactor( red, luminanceScaled, redDeltaScaled, scaleFactor ) ) {
		scaleAdjusted = true;
	}
	if( adjustScaleFactor( green, luminanceScaled, greenDeltaScaled, scaleFactor ) ) {
		scaleAdjusted = true;
	}
	if( adjustScaleFactor( blue, luminanceScaled, blueDeltaScaled, scaleFactor ) ) {
		scaleAdjusted = true;
	}
	if( scaleAdjusted ) {
		red = ( luminanceScaled + Round( scaleFactor * redDeltaScaled ) + CoeffNormalization / 2 ) >> CoeffNormalizationBitsCount;
		green = ( luminanceScaled + Round( scaleFactor * greenDeltaScaled ) + CoeffNormalization / 2 ) >> CoeffNormalizationBitsCount;
		blue = ( luminanceScaled + Round( scaleFactor * blueDeltaScaled ) + CoeffNormalization / 2 ) >> CoeffNormalizationBitsCount;
	}
	//  ,   max(0,min) -  ,    2   .
	//   ,      .
	rgb.Red() = to<TColorComponent>( red < 0 ? 0 : ( red > ColorComponentMax ? ColorComponentMax : red ) );
	rgb.Green() = to<TColorComponent>( green < 0 ? 0 : ( green > ColorComponentMax ? ColorComponentMax : green ) );
	rgb.Blue() = to<TColorComponent>( blue < 0 ? 0 : ( blue > ColorComponentMax ? ColorComponentMax : blue ) );
}

//  YCC  RGB,    
inline void fromYCCToRGB( const CYCbCrColor& ycc, CRGBColor& rgb )
{
	double scaleFactor = 1.0;
	fromYCCToRGB( ycc, rgb, scaleFactor );
}

//  YCC  RGB,   RGB    0..255,  
//  .
inline void fromYCCToRGBClipComponents( const CYCbCrColor& ycc, CRGBColor& rgb )
{
	const int chrominanceBlueSigned = ycc.ChrominanceBlue() - 128;
	const int chrominanceRedSigned = ycc.ChrominanceRed() - 128;
	const int luminanceScaled = ycc.Luminance() * CoeffNormalization;

	const int redDeltaScaled = CoeffYcc2RgbRedCr * chrominanceRedSigned;
	const int greenDeltaScaled = CoeffYcc2RgbGreenCb * chrominanceBlueSigned + CoeffYcc2RgbGreenCr * chrominanceRedSigned;
	const int blueDeltaScaled = CoeffYcc2RgbBlueCb * chrominanceBlueSigned;

	int red = ( luminanceScaled + redDeltaScaled + CoeffNormalization / 2 ) >> CoeffNormalizationBitsCount;
	int green = ( luminanceScaled + greenDeltaScaled + CoeffNormalization / 2 ) >> CoeffNormalizationBitsCount;
	int blue = ( luminanceScaled + blueDeltaScaled + CoeffNormalization / 2 ) >> CoeffNormalizationBitsCount;

	rgb.Red() = to<TColorComponent>( red < 0 ? 0 : ( red > ColorComponentMax ? ColorComponentMax : red ) );
	rgb.Green() = to<TColorComponent>( green < 0 ? 0 : ( green > ColorComponentMax ? ColorComponentMax : green ) );
	rgb.Blue() = to<TColorComponent>( blue < 0 ? 0 : ( blue > ColorComponentMax ? ColorComponentMax : blue ) );
}

//	   RGB   HSL -> RGB
inline TColorComponent getRGBComponentFromHSL( int p, int q, int tC ) 
{
	if( tC < 0 ) { 
		tC += CoeffNormalization;
	} else if( tC > CoeffNormalization ) {
		tC -= CoeffNormalization;
	}

	static const int oneSixth =  CoeffNormalization / 6;
	static const int oneHalf =  CoeffNormalization / 2;
	static const int twoThirds = 2 * CoeffNormalization / 3;

	int value = 0;
	if( tC < oneSixth ) {
		value = p + ( ( (q - p) * 6 * tC ) >> CoeffNormalizationBitsCount );
	} else if( tC < oneHalf ) {
		value = q;
	} else if( tC < twoThirds ) {
		value = p + ( ( (q - p) * 6 * ( twoThirds - tC ) ) >> CoeffNormalizationBitsCount );
	} else {
		value = p;
	}
	presume( value >= 0 && value <= CoeffNormalization );
	return to<TColorComponent>( (value * 255 + CoeffNormalization / 2) >> CoeffNormalizationBitsCount );
}

//	 RGB   ,     HSL.
//	minMaxSum, minMaxDelta -        r, g, b ;
//	hue -  Hue.
inline void getRGBtoHSLParams( const CRGBColor& rgb, 
	int& minMaxSum, int& minMaxDelta, TColorComponent& hue )
{
	int delta12 = 0;	//	    ,    
	int hslCenterValue = 0;	//	  Hue (  256/6)

	//	   6   (r, g, b)  
	if( rgb.Red() >= rgb.Blue() ) {
		if( rgb.Red() >= rgb.Green() ) {
			if( rgb.Blue() >= rgb.Green() ) {
				//	g, b, r
				minMaxDelta = rgb.Red() - rgb.Green();
				minMaxSum = rgb.Red() + rgb.Green();
				delta12 = rgb.Green() - rgb.Blue();
				hslCenterValue = 6;
			} else {
				//	b, g, r
				minMaxDelta = rgb.Red() - rgb.Blue();
				minMaxSum = rgb.Red() + rgb.Blue();
				delta12 = rgb.Green() - rgb.Blue();
				hslCenterValue = 0;
			}
		} else {
			//	b, r, g
			minMaxDelta = rgb.Green() - rgb.Blue();
			minMaxSum = rgb.Green() + rgb.Blue();
			delta12 = rgb.Blue() - rgb.Red();
			hslCenterValue = 2;
		}
	} else {
		if( rgb.Red() >= rgb.Green() ) {
			//	g, r, b
			minMaxDelta = rgb.Blue() - rgb.Green();
			minMaxSum = rgb.Blue() + rgb.Green();
			delta12 = rgb.Red() - rgb.Green();
			hslCenterValue = 4;
		} else {
			if( rgb.Blue() >= rgb.Green() ) {
				//	r, g, b
				minMaxDelta = rgb.Blue() - rgb.Red();
				minMaxSum = rgb.Blue() + rgb.Red();
				delta12 = rgb.Red() - rgb.Green();
				hslCenterValue = 4;
			} else {
				// r, b, g
				minMaxDelta = rgb.Green() - rgb.Red();
				minMaxSum = rgb.Green() + rgb.Red();
				delta12 = rgb.Blue() - rgb.Red();
				hslCenterValue = 2;
			}
		}
	}

	hue = minMaxDelta == 0 ? 0 : to<TColorComponent>( 
		( (delta12 * RgbToHslFactor) / minMaxDelta + hslCenterValue * RgbToHslFactor ) >> CoeffNormalizationBitsCount );
}


inline void fromRGBToHSLNormalized( const CRGBColor& rgb, CHSLColorNormalized& hsl )
{
	int minMaxSum = 0;
	int minMaxDelta = 0;
	getRGBtoHSLParams( rgb, minMaxSum, minMaxDelta, hsl.Hue() );

	hsl.Lightness() = to<TColorComponent>( minMaxSum >> 1 );
	if( minMaxDelta == 0 ) {
		hsl.SaturationNormalized() = 0;
	} else {
		const int denom = minMaxSum <= 255 ? minMaxSum : 2 * 255 - minMaxSum;
		presume( denom > 0 );	//	 minMaxSum == 0  minMaxSum == 2 * 255,   minMaxDelta == 0.
		hsl.SaturationNormalized() = to<TColorComponent>( (255 * minMaxDelta + denom / 2) / denom );
	}
}

inline void fromRGBToHSL( const CRGBColor& rgb, CHSLColorN& hsl )
{
	int minMaxSum = 0;
	int minMaxDelta = 0;
	getRGBtoHSLParams( rgb, minMaxSum, minMaxDelta, hsl.Hue() );

	hsl.Lightness() = to<TColorComponent>( minMaxSum >> 1 );
	hsl.Saturation() = minMaxDelta == 0 ? 0 : to<TColorComponent>( minMaxDelta );
}

inline void fromHSLNormalizedToRGB( const CHSLColorNormalized& hsl, CRGBColor& rgb )
{
	//	  CoeffNormalization 
	const int h = ( hsl.Hue() << CoeffNormalizationBitsCount ) / 256;	//	256,   . h  [0, 1),    s, l,   [0,1]
	const int s = ( hsl.SaturationNormalized() << CoeffNormalizationBitsCount ) / 255;
	const int l = ( hsl.Lightness() << CoeffNormalizationBitsCount ) / 255;

	//	    ,  h, s, l <= CoeffNormalization,     
	const int q = l < CoeffNormalization / 2 ? l * (CoeffNormalization + s) >> CoeffNormalizationBitsCount :
		l + s - ( l * s >> CoeffNormalizationBitsCount );
	const int p = 2 * l - q;

	static const int  oneThird = CoeffNormalization / 3;
	rgb.Red() = getRGBComponentFromHSL( p, q, h + oneThird );
	rgb.Green() = getRGBComponentFromHSL( p, q, h );
	rgb.Blue() = getRGBComponentFromHSL( p, q, h - oneThird );
}

inline void fromHSLToRGB( const CHSLColorN& hsl, CRGBColor& rgb )
{
	//	  CoeffNormalization 
	const int h = ( hsl.Hue() << CoeffNormalizationBitsCount ) / 256;	//	256,   . h  [0, 1),    s, l,   [0,1]
	const int s = ( hsl.Saturation() << CoeffNormalizationBitsCount ) / 255;
	const int l = ( hsl.Lightness() << CoeffNormalizationBitsCount ) / 255;

	const int q = l + s  / 2;
	const int p = 2 * l - q;

	static const int  oneThird = CoeffNormalization / 3;
	rgb.Red() = getRGBComponentFromHSL( p, q, h + oneThird );
	rgb.Green() = getRGBComponentFromHSL( p, q, h );
	rgb.Blue() = getRGBComponentFromHSL( p, q, h - oneThird );
}

inline void fromHSLToHSLNormalized( const CHSLColorN& hsl, CHSLColorNormalized& hslNormlaized )
{
	// HSL -> HSL   
	hslNormlaized.Hue() = hsl.Hue();
	hslNormlaized.Lightness() = hsl.Lightness();
	const int denom = hsl.Lightness() <= 128 ? 2 * hsl.Lightness() : 2 * ( 255 - hsl.Lightness() );
	int saturation = denom == 0 ? 0 : ( 255 * hsl.Saturation() + denom / 2 ) / denom;
	//	    
	saturation = min( ColorComponentMax, saturation );
	hslNormlaized.SaturationNormalized() = to<TColorComponent>( saturation );
}

inline void fromHSLNormalizedToHSL( const CHSLColorNormalized& hslNormalized, CHSLColorN& hsl )
{
	// HSL    -> HSL
	hsl.Hue() = hslNormalized.Hue();
	hsl.Lightness() = hslNormalized.Lightness();
	hsl.Saturation() = hslNormalized.Lightness() <= 128 ? 
		to<TColorComponent>( Round( hslNormalized.SaturationNormalized() * hslNormalized.Lightness(), 128 ) ) : 
		to<TColorComponent>( Round( hslNormalized.SaturationNormalized() * (255 - hslNormalized.Lightness()), 128 ) );
}

inline void fromRGBToCMYK( const CRGBColor& rgb, CCMYKColor& cmyk )
{
	//	      RGB->CMYK,  
	//	  fromCMYKToRGB (.    fromCMYKToRGB)
	const int cyan = ColorComponentMax - rgb.Red();
	const int magenta = ColorComponentMax - rgb.Green();
	const int yellow = ColorComponentMax - rgb.Blue();
	const int key = min( min( cyan, magenta ), yellow );
	presume( key >= 0 && key <= ColorComponentMax );
	cmyk.Key() = to<TColorComponent>( key );
	if( key == ColorComponentMax ) {
		cmyk.Cyan() = cmyk.Magenta() = cmyk.Yellow() = 0;
	} else {
		cmyk.Cyan() = to<TColorComponent>( (( cyan - key ) * ColorComponentMax) / (ColorComponentMax - key) );
		cmyk.Magenta() = to<TColorComponent>( (( magenta - key ) * ColorComponentMax) / (ColorComponentMax - key) );
		cmyk.Yellow() = to<TColorComponent>( (( yellow - key ) * ColorComponentMax) / (ColorComponentMax - key) );
	}
 }

inline void fromCMYKToRGB( const CCMYKColor& cmyk, CRGBColor& rgb )
{
	rgb.Red() = to<TColorComponent>( ( (ColorComponentMax - cmyk.Cyan()) * (ColorComponentMax - cmyk.Key()) ) / ColorComponentMax );
	rgb.Green() = to<TColorComponent>( ( (ColorComponentMax - cmyk.Magenta()) * (ColorComponentMax - cmyk.Key()) ) / ColorComponentMax );
	rgb.Blue() = to<TColorComponent>( ( (ColorComponentMax - cmyk.Yellow()) * (ColorComponentMax - cmyk.Key()) ) / ColorComponentMax );

	//	CMYK - - ,       RGB
	//	   :
		
	//	Standart -    CMYK -> RGB     
	//	     GhostScript
	//	       ,   .
	//	red = ( (255 - cyan) * (255 - key) ) / 255
	//	green = ( (255 - magenta) * (255 - key) ) / 255
	//	blue = ( (255 - yellow) * (255 - key) ) / 255

	//	Adobe - ,        Adobe
	//	 ,    
	//	red = 255 - min( 255, cyan + key )
	//	green = 255 - min( 255, magenta + key )
	//	blue = 255 - min( 255, yellow + key )

	//	ABBYY -    FineReader 8.0  SDK 8.0
	//	     ,   
	//	red = 255 - cyan
	//	green = 255 - magenta
	//	blue = 255 - yellow
}

inline void fromRGBToYCCK( const CRGBColor& rgb, CYCbCrKColor& ycck )
{
	CCMYKColor cmyk;
	fromRGBToCMYK( rgb, cmyk );
	fromCMYKToYCCK( cmyk, ycck );
}

inline void fromYCCKToRGB( const CYCbCrKColor& ycck, CRGBColor& rgb )
{
	CCMYKColor cmyk;
	fromYCCKToCMYK( ycck, cmyk );
	fromCMYKToRGB( cmyk, rgb );
}

inline void fromYCCToCMYK( const CYCbCrColor& ycc, CCMYKColor& cmyk )
{
	CRGBColor rgb;
	fromYCCToRGB( ycc, rgb );
	fromRGBToCMYK( rgb, cmyk );
}

inline void fromCMYKToYCC( const CCMYKColor& cmyk, CYCbCrColor& ycc )
{
	CRGBColor rgb;
	fromCMYKToRGB( cmyk, rgb );
	fromRGBToYCC( rgb, ycc );
}

inline void fromYCCToYCCK( const CYCbCrColor& ycc, CYCbCrKColor& ycck )
{
	CRGBColor rgb;
	fromYCCToRGB( ycc, rgb );
	CCMYKColor cmyk;
	fromRGBToCMYK( rgb, cmyk );
	fromCMYKToYCCK( cmyk, ycck );
}

inline void fromYCCKToYCC( const CYCbCrKColor& ycck, CYCbCrColor& ycc )
{
	CCMYKColor cmyk;
	fromYCCKToCMYK( ycck, cmyk );
	CRGBColor rgb;
	fromCMYKToRGB( cmyk, rgb );
	fromRGBToYCC( rgb, ycc );
}

inline void fromCMYKToYCCK( const CCMYKColor& cmyk, CYCbCrKColor& ycck )
{
	//	-1 ,  255.5   255,  0.5   0
	static const int chrominanceAddition = 128 * CoeffNormalization + CoeffNormalization / 2 - 1;

	const int red = ColorComponentMax - cmyk.Cyan();
	const int green = ColorComponentMax - cmyk.Magenta();
	const int blue = ColorComponentMax - cmyk.Yellow();

	const int luminance = ( ( LumaRed * red + LumaGreen * green + LumaBlue * blue + CoeffNormalization / 2 ) >> 
		CoeffNormalizationBitsCount );

	const int chrominanceBlue = ( ( CoeffRgb2YccCbRed * red + CoeffRgb2YccCbGreen * green + 
		CoeffRgb2YccCbBlue * blue + chrominanceAddition ) >> CoeffNormalizationBitsCount );
	const int chrominanceRed = ( ( CoeffRgb2YccCrRed * red + CoeffRgb2YccCrGreen * green + 
		CoeffRgb2YccCrBlue * blue + chrominanceAddition ) >> CoeffNormalizationBitsCount );

	ycck.Luminance() = to<TColorComponent>( luminance );
	ycck.ChrominanceBlue() = to<TColorComponent>( chrominanceBlue );
	ycck.ChrominanceRed() = to<TColorComponent>( chrominanceRed );

	ycck.Key() = cmyk.Key();
}

inline void fromYCCKToCMYK( const CYCbCrKColor& ycck, CCMYKColor& cmyk )
{
	CYCbCrColor ycc;
	CRGBColor rgb;
	ycc.Luminance() = ycck.Luminance();
	ycc.ChrominanceRed() = ycck.ChrominanceRed();
	ycc.ChrominanceBlue() = ycck.ChrominanceBlue();
	fromYCCToRGB( ycc, rgb );
	
	cmyk.Cyan() = ColorComponentMax - rgb.Red();
	cmyk.Magenta() = ColorComponentMax - rgb.Green();
	cmyk.Yellow() = ColorComponentMax - rgb.Blue();
	cmyk.Key() = ycck.Key();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

inline void fromRGBToGray( const CRGBColor& rgb, CGrayColor& gray )
{
	gray.Value() = ColorComponentMax - fromRGBToLuminance( rgb );
}

inline void fromRGBToGray( const TColorComponent red, const TColorComponent green, const TColorComponent blue, CGrayColor& gray )
{
	gray.Value() = ColorComponentMax - fromRGBToLuminance( red, green, blue );
}

inline void fromYCCToGray( const CYCbCrColor& ycc, CGrayColor& gray )
{
	gray.Value() = ColorComponentMax - ycc.Luminance();
}

inline void fromCMYKToGray( const CCMYKColor& cmyk, CGrayColor& gray )
{
	CRGBColor rgb;
	fromCMYKToRGB( cmyk, rgb );
	fromRGBToGray( rgb, gray );
}

inline void fromYCCKToGray( const CYCbCrKColor& ycck, CGrayColor& gray )
{
	CCMYKColor cmyk;
	fromYCCKToCMYK( ycck, cmyk );
	CRGBColor rgb;
	fromCMYKToRGB( cmyk, rgb );
	fromRGBToGray( rgb, gray );
}

inline void fromGrayToRGB( const CGrayColor& gray, CRGBColor& rgb )
{
	rgb.Red() = rgb.Green() = rgb.Blue() = ColorComponentMax - gray.Value();
}

inline void fromGrayToYCC( const CGrayColor& gray, CYCbCrColor& ycc )
{
	ycc.Luminance() = ColorComponentMax - gray.Value();
	ycc.ChrominanceBlue() = ycc.ChrominanceRed() = 128;
}

inline void fromGrayToCMYK( const CGrayColor& gray, CCMYKColor& cmyk )
{
	cmyk.Cyan() = cmyk.Magenta() = cmyk.Yellow() = 0;
	cmyk.Key() = gray.Value();
}

inline void fromGrayToYCCK( const CGrayColor& gray, CYCbCrKColor& ycck )
{
	ycck.Luminance() = ColorComponentMax;
	ycck.ChrominanceBlue() = ycck.ChrominanceRed() = 128;
	ycck.Key() = gray.Value();
}

inline TColorComponent fromRGBToLuminance( const CRGBColor& rgb )
{
	return to<TColorComponent>( ( LumaRed * rgb.Red() + LumaGreen * rgb.Green() + LumaBlue * rgb.Blue() + CoeffNormalization / 2 ) >> 
		CoeffNormalizationBitsCount );
}

inline TColorComponent fromRGBToLuminance( const TColorComponent red, const TColorComponent green, const TColorComponent blue )
{
	return to<TColorComponent>( ( LumaRed * red + LumaGreen * green + LumaBlue * blue + CoeffNormalization / 2 ) >> 
		CoeffNormalizationBitsCount );
}

inline void fromRGBToHSV( const CRGBColor& rgb, CHSVColor& hsv )
{
	const int red = rgb.Red();
	const int green = rgb.Green();
	const int blue = rgb.Blue();

	const int maxRGB = max( red, max( green, blue ) );
	const int minRGB = min( red, min( green, blue ) );

	if( minRGB < maxRGB ) {
		const int diff = maxRGB - minRGB;

		const int s = diff * ColorComponentMax / maxRGB;
		presume( 0 <= s && s <= ColorComponentMax );

		const int v = ( red + green + blue ) / 3;

		const int denominator = 6 * diff;

		int h = 0;
		if( maxRGB == red ) {
			if( minRGB == green ) {
				h = ( 5 * diff + maxRGB - blue ) * ColorComponentMax / denominator;
			} else {
				h = ( diff - maxRGB + green ) * ColorComponentMax / denominator;
			}
		} else if( maxRGB == green ) {
			if( minRGB == blue ) {
				h = ( diff + maxRGB - red ) * ColorComponentMax / denominator;
			} else {
				h = ( 3 * diff - maxRGB + blue ) * ColorComponentMax / denominator;
			}
		} else {
			if( minRGB == red ) {
				h = ( 3 * diff + maxRGB - green ) * ColorComponentMax / denominator;
			} else {
				h = ( 5 * diff - maxRGB + red ) * ColorComponentMax / denominator;
			}
		}
		presume( 0 <= h && h <= ColorComponentMax );

		hsv.Hue() = to<TColorComponent>( h );
		hsv.Saturation() = to<TColorComponent>( s );
		hsv.Value() = to<TColorComponent>( v );
	} else { //  
		hsv.Hue() = 0;
		hsv.Saturation() = 0;
		hsv.Value() = to<TColorComponent>( maxRGB );
	}
}

inline void fromHSVToRGB( const CHSVColor& hsv, CRGBColor& rgb )
{
	if( hsv.Saturation() == 0 ) { //  
		rgb.Red() = hsv.Value();
		rgb.Green() = hsv.Value();
		rgb.Blue() = hsv.Value();

		return;
	}

	double h = (double) hsv.Hue() / (double) ColorComponentMax;
	double s = (double) hsv.Saturation() / (double) ColorComponentMax;
	double v = (double) hsv.Value() / (double) ColorComponentMax;

	h = 6 * h;
	int i = Floor( h );
	double f = h - i;

	double m = v * ( 1 - s );
	double n = v * ( 1 - s * f );
	double k = v * ( 1 - s * ( 1 - f ) );

	double r = 0, g = 0, b = 0;
	switch( i ) {
		case 0:
		case 6:
		{
			r = v;
			g = k;
			b = m;
		}
			break;
		case 1:
		{
			r = n;
			g = v;
			b = m;
		}
			break;
		case 2:
		{
			r = m;
			g = v;
			b = k;
		}
			break;
		case 3:
		{
			r = m;
			g = n;
			b = v;
		}
			break;
		case 4:
		{
			r = k;
			g = m;
			b = v;
		}
			break;
		case 5:
		{
			r = v;
			g = m;
			b = n;
		}
			break;
		default:
			assert( false );
	}

	rgb.Red() = to<TColorComponent>( (int) ( r * ColorComponentMax ) );
	rgb.Green() = to<TColorComponent>( (int) ( g * ColorComponentMax ) );
	rgb.Blue() = to<TColorComponent>( (int) ( b * ColorComponentMax ) );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

inline CColorValue::CColorValue( const CColorValue& other )
{
	SetColor( other );
}

inline CColorValue::CColorValue( COLORREF color ) 
{
	SetColor( color );
}

inline CColorValue::CColorValue( TBWColor color )
{
	SetColor( color );
}

inline CColorValue::CColorValue( TColorComponent grayValue )
{
	SetColor( grayValue );
}

inline CColorValue::CColorValue( TColorComponent red, TColorComponent green, TColorComponent blue ) 	
{
	SetColor( red, green, blue );
}

inline CColorValue::CColorValue( const CGrayColor& color )
{
	SetColor( color );
}

inline CColorValue::CColorValue( const CRGBColor& color )	
{
	SetColor( color );
}


inline void CColorValue::SetColor( const CColorValue& color )
{
	rgbColor = color.rgbColor;
	grayColor = color.grayColor;
	bwColor = color.bwColor;
	colorRef = color.colorRef;
}

inline void CColorValue::SetColor( COLORREF color )
{
	colorRef = color;
	rgbColor.Red() = GetRValue( colorRef );
	rgbColor.Green() = GetGValue( colorRef );
	rgbColor.Blue() = GetBValue( colorRef );
	fromRGBToGray( rgbColor, grayColor );
	bwColor = ( grayColor.Value() >= ColorComponentMax / 2 ? BWC_Black : BWC_White );
}

inline void CColorValue::SetColor( TBWColor color )
{
	bwColor = color;
	grayColor.Value() = ( bwColor == BWC_Black ? ColorComponentMax : 0 );
	fromGrayToRGB( grayColor, rgbColor );
	colorRef = RGB( rgbColor.Red(), rgbColor.Green(), rgbColor.Blue() );
}

inline void CColorValue::SetColor( TColorComponent grayValue )
{
	grayColor.Value() = grayValue;
	fromGrayToRGB( grayColor, rgbColor );
	colorRef = RGB( rgbColor.Red(), rgbColor.Green(), rgbColor.Blue() );
	bwColor = ( grayColor.Value() >= ColorComponentMax / 2 ? BWC_Black : BWC_White );
}

inline void CColorValue::SetColor( TColorComponent red, TColorComponent green, TColorComponent blue )
{
	colorRef = RGB( red, green, blue );
	rgbColor.Red() = red;
	rgbColor.Green() = green;
	rgbColor.Blue() = blue;
	fromRGBToGray( rgbColor, grayColor );
	bwColor = ( grayColor.Value() >= ColorComponentMax / 2 ? BWC_Black : BWC_White );
}

inline void CColorValue::SetColor( const CGrayColor& color )
{
	grayColor = color;
	fromGrayToRGB( grayColor, rgbColor );
	colorRef = RGB( rgbColor.Red(), rgbColor.Green(), rgbColor.Blue() );
	bwColor = ( grayColor.Value() >= ColorComponentMax / 2 ? BWC_Black : BWC_White );
}

inline void CColorValue::SetColor( const CRGBColor& color )
{
	rgbColor = color;
	colorRef = RGB( rgbColor.Red(), rgbColor.Green(), rgbColor.Blue() );
	fromRGBToGray( rgbColor, grayColor );
	bwColor = ( grayColor.Value() >= ColorComponentMax / 2 ? BWC_Black : BWC_White );
}

template<>
inline CRGBColor CColorValue::GetValue() const 
{ 
	return rgbColor; 
}

template<>
inline CGrayColor CColorValue::GetValue() const 
{ 
	return grayColor; 
}

template<>
inline TBWColor CColorValue::GetValue() const 
{ 
	return bwColor; 
}

inline bool CColorValue::operator == ( const CColorValue& other ) const
{
	return colorRef == other.colorRef;
}

inline bool CColorValue::operator != ( const CColorValue& other ) const
{
	return colorRef != other.colorRef;
}

inline CArchive& operator << ( CArchive& archive, const CColorValue& color )
{
	archive.Write( &color.colorRef, sizeof( color.colorRef ) );
	return archive;
}

inline CArchive& operator >> ( CArchive& archive, CColorValue& color )
{
	COLORREF cr;

	archive.Read( &cr, sizeof( cr ) );
	color.SetColor( cr );

	return archive;
}



};	//	namespace Image.
