#pragma once
#include <Array.h>
#include <ExtraNeoML/TraditionalML/Utilities.h>

namespace Image {
namespace FastCrop {

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//    IModel-, , , 
inline void serializeNullableIModel( CPtr<NeoML::IModel>& model, CArchive& arc )
{
	if( arc.IsLoading() ) {
		LoadFmlModel( arc, model );
	} else {
		arc.WriteObject( model );
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CPreprocessingParams
inline void CPreprocessingParams::Serialize( CArchive& arc )
{
	arc.SerializeVersion( 0 );

	arc.Serialize( GrayChannelPriority );
	arc.Serialize( LogStretchedSize );
	arc.Serialize( MedianIterationCount );
	arc.Serialize( LowCannyThreshold );
	arc.Serialize( HighCannyThreshold );
	arc.Serialize( CannyEnhance );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CLineFinderParams
inline void CLineFinderParams::Serialize( CArchive& arc )
{
	arc.SerializeVersion( 0 );

	arc.Serialize( LineHypothesesCount );
	arc.Serialize( FeaturePointsRadius );
	arc.Serialize( LineImagePart );
	SerializeModel( arc, LinesClassifierModel );
	arc.Serialize( SimilarityThreshold );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CQuadrangleGeometryFeatureRanges
inline void CQuadrangleGeometryFeatureRanges::Serialize( CArchive& arc )
{
	arc.SerializeVersion( 0 );

	arc.Serialize( AreaRange.Begin );
	arc.Serialize( AreaRange.End );
	arc.Serialize( SideLengthRange.Begin );
	arc.Serialize( SideLengthRange.End );
	arc.Serialize( AngleRange.Begin );
	arc.Serialize( AngleRange.End );
	arc.Serialize( AspectRatioRange.Begin );
	arc.Serialize( AspectRatioRange.End );
	arc.Serialize( OppositeSidesRatioRange.Begin );
	arc.Serialize( OppositeSidesRatioRange.End );
	arc.Serialize( ConvergenceRange.Begin );
	arc.Serialize( ConvergenceRange.End );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CCornerImagesGeneratorParams
inline void CCornerImagesGeneratorParams::Serialize( CArchive& arc )
{
	arc.SerializeVersion( 0 );

	arc.Serialize( InsideCornerSurrounding );
	arc.Serialize( OutsideCornerSurrounding );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CFixedCnnParams
inline void CFixedCnnParams::Serialize( CArchive& arc )
{
	arc.SerializeVersion( 0 );

	Conv1Coeff.Serialize( arc );
	Conv1Bias.Serialize( arc );
	Conv2Coeff.Serialize( arc );
	Conv2Bias.Serialize( arc );
	Conv3Coeff.Serialize( arc );
	Conv3Bias.Serialize( arc );
	Dense1Coeff.Serialize( arc );
	Dense1Bias.Serialize( arc );
	Dense2Coeff.Serialize( arc );
	Dense2Bias.Serialize( arc );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CQuadrangleFinderParams
inline void CQuadrangleFinderParams::Serialize( CArchive& arc )
{
	arc.SerializeVersion( 0 );

	arc.Serialize( SrcHypothesesCount );
	arc.Serialize( MinOppositeSidesDistance );
	GeometryFeatureRanges.Serialize( arc );
	SerializeModel( arc, FirstLevelClassifier );
	arc.Serialize( FirstLevelClassifierThreshold );
	arc.Serialize( FirstLevelSimilarityThreshold );
	arc.Serialize( SecondLevelHypothesesCount );
	CornerImagesParams.Serialize( arc );
	CornerDetectionCnnWeights.Serialize( arc );
	SerializeModel( arc, SecondLevelClassifier );
	arc.Serialize( SecondLevelClassifierThreshold );
	arc.Serialize( SecondLevelSimilarityThreshold );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CClassifierFPParams
inline void CFalsePositiveClassifierParams::Serialize( CArchive& arc )
{
	arc.SerializeVersion( 0 );

	SerializeModel( arc, FPClassifier );
	arc.Serialize( TruthProbabilityThreshold );
	SerializeModel( arc, JaccardClassifier );
	arc.Serialize( SpreadIndex );
}

inline void CFastCropModel::Serialize( CArchive& arc )
{
	arc.SerializeVersion( 0 );

	LineFinderParams.Serialize( arc );
	PreprocessingParams.Serialize( arc );
	QuadrangleFinderParams.Serialize( arc );
	FalsePositiveClassifierParams.Serialize( arc );
}

} // namespace FastCrop
} // namespace Image

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
namespace FObj {

inline void MoveTo( Image::FastCrop::CFastCropLine& from, Image::FastCrop::CFastCropLine& to )
{
	to.Line = from.Line;
	to.Point = from.Point;
	to.Weight = from.Weight;

	from.CumulativeWeight.MoveTo( to.CumulativeWeight );
	from.CumulativeQuality.MoveTo( to.CumulativeQuality );

	to.Center = from.Center;
	to.Angle = from.Angle;
	from.HoughFeaturePointWeights.MoveTo( to.HoughFeaturePointWeights );
	to.BeginContrast = from.BeginContrast;
	to.MidContrast = from.MidContrast;
	to.EndContrast = from.EndContrast;
	to.Quality = from.Quality;
}

template<>
inline void ArrayMemMoveElement<Image::FastCrop::CFastCropLine>( Image::FastCrop::CFastCropLine* dest, Image::FastCrop::CFastCropLine* source )
{
	presume( dest != source );
	::new( dest )Image::FastCrop::CFastCropLine();
	MoveTo( *source, *dest );
}

template<>
inline void SimpleSwap<Image::FastCrop::CFastCropLine>( Image::FastCrop::CFastCropLine& a, Image::FastCrop::CFastCropLine& b )
{
	Image::FastCrop::CFastCropLine c;
	MoveTo( a, c );
	MoveTo( b, a );
	MoveTo( c, b );
}

} // namespace FObj