
//	Copyright (c) ABBYY (BIT Software).
//	:		
//	:	FineReader
//	:	Image
//	:	 inline-,   <Image.ImageInfo.h>.

namespace Image {

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

inline CSize IImageInfo::GetSize() const
{
	const int width = GetWidth();
	const int height = GetHeight();

	return CSize( width, height );
}

inline CRect IImageInfo::GetRect() const
{
	const int width = GetWidth();
	const int height = GetHeight();

	return CRect( 0, 0, width, height );
}

inline int IImageInfo::GetResolution() const
{
	const int xResolution = GetXResolution();
	const int yResolution = GetYResolution();
	assert( xResolution == yResolution );

	return xResolution;
}

inline bool IImageInfo::IsIsotropic() const
{
	const int xResolution = GetXResolution();
	const int yResolution = GetYResolution();
	return xResolution == yResolution;
}

inline bool IImageInfo::IsPhoto() const
{
	CPtr<const IMetadata> metadata = GetMetadata();
	if( metadata == 0 ) {
		return false;
	}
	CPtr<const IMetadataEntry> entry = metadata->GetEntry( MetadataIsPhoto );
	int entryValue = 0;
	return ( entry != 0 && entry->GetData( entryValue ) && entryValue != 0 );
}

inline bool IImageInfo::IsCertainPhoto() const
{
	CPtr<const IMetadata> metadata = GetMetadata();
	if( metadata == 0 ) {
		return false;
	}
	CPtr<const IMetadataEntry> entry = metadata->GetEntry( MetadataIsCertainPhoto );
	int entryValue = 0;
	return ( entry != 0 && entry->GetData( entryValue ) && entryValue != 0 );
}

inline bool IImageInfo::IsCertainScan() const
{
	CPtr<const IMetadata> metadata = GetMetadata();
	if( metadata == 0 ) {
		return false;
	}
	CPtr<const IMetadataEntry> entry = metadata->GetEntry( MetadataIsCertainScan );
	int entryValue = 0;
	return ( entry != 0 && entry->GetData( entryValue ) && entryValue != 0 );
}


inline bool IImageInfo::IsPDFWithTextLayer() const
{
	CPtr<const IMetadata> metadata = GetMetadata();
	if( metadata == 0 ) {
		return false;
	}
	CPtr<const IMetadataEntry> entry = metadata->GetEntry( MetadataIsPDFWithTextLayer );
	int entryValue = 0;
	return ( entry != 0 && entry->GetData( entryValue ) && entryValue != 0 );
}

inline TImageFileFormat IImageInfo::GetSourceFileFormat() const
{
	CPtr<const IMetadata> metadata = GetMetadata();
	if( metadata == 0 ) {
		return IFF_Unknown;
	}
	CPtr<const IMetadataEntry> entry = metadata->GetEntry( MetadataSourceFileFormatPath );
	int format = 0;
	if( entry == 0 || !entry->GetData( format ) ) {
		return IFF_Unknown;
	}
	return to<TImageFileFormat>( format );
}

inline bool IImageInfo::IsImageFromDocument() const
{
	CPtr<const IMetadata> metadata = GetMetadata();
	if( metadata == 0 ) {
		return false;
	}
	//	   MetadataDocumentTagPath
	return metadata->GetEntry( MetadataDocumentTagPath ) != 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//

};	//	namespace Image.
