//	Copyright (c) ABBYY (BIT Software).
//	:	 inline-,   <ImageTransformation.h>.

#include <Image.ImageTransformer.h>
#include <Image.TransformationFunctions.h>
#include <Image.StretchParams.h>
#include <Image.ScaleDownTwiceParams.h>

namespace Image {

inline CSize IImageTransformationServices::GetStretchedImageSize( const CSize& originalSize,
	double stretchRatio ) const
{
	CPtr<IStretchParams> params = FObj::CreateObject<IStretchParams>( StretchParamsClass );
	params->SetStretchRatio( stretchRatio );

	CPtr<IImageTransformer> stretchTransformer = FObj::CreateObject<IImageTransformer>(
		StretchTransformerClass );
	CSize transformedSize;
	stretchTransformer->CalculateTransformedImageSize( originalSize, params, transformedSize );

	return transformedSize;
}

//	       ScaleDownImageTwice .
inline CSize IImageTransformationServices::GetScaledDownTwiceImageSize(
	const CSize& originalSize ) const
{
	CPtr<IScaleDownTwiceParams> params = FObj::CreateObject<IScaleDownTwiceParams>(
		ScaleDownTwiceParamsClass );
	CPtr<IImageTransformer> transformer = FObj::CreateObject<IImageTransformer>(
		ScaleDownTwiceTransformerClass );

	CSize transformedSize;
	transformer->CalculateTransformedImageSize( originalSize, params, transformedSize );
	return transformedSize;
}

//	       StretchImageTwice .
inline CSize IImageTransformationServices::GetStretchedTwiceImageSize( const CSize& originalSize ) const
{
	CPtr<IImageTransformer> transformer = FObj::CreateObject<IImageTransformer>(
		StretchTwiceTransformerClass );

	CSize transformedSize;
	transformer->CalculateTransformedImageSize( originalSize, 0, transformedSize );
	return transformedSize;
}

} // namespace Image